@@ -11,6 +11,7 @@ import {
   stringMatch,
   stringReplace,
   stringToLowerCase,
+  stringToString,
   stringIndexOf,
   stringTrim,
   regExpTest,
@@ -161,6 +162,7 @@ function createDOMPurify(window = getGlobal()) {
   const {
     MUSTACHE_EXPR,
     ERB_EXPR,
+    TMPLIT_EXPR,
     DATA_ATTR,
     ARIA_ATTR,
     IS_SCRIPT_OR_DATA,
@@ -364,6 +366,14 @@ function createDOMPurify(window = getGlobal()) {
   let NAMESPACE = HTML_NAMESPACE;
   let IS_EMPTY_INPUT = false;
 
+  /* Allowed XHTML+XML namespaces */
+  let ALLOWED_NAMESPACES = null;
+  const DEFAULT_ALLOWED_NAMESPACES = addToSet(
+    {},
+    [MATHML_NAMESPACE, SVG_NAMESPACE, HTML_NAMESPACE],
+    stringToString
+  );
+
   /* Parsing of strict XHTML documents */
   let PARSER_MEDIA_TYPE;
   const SUPPORTED_PARSER_MEDIA_TYPES = ['application/xhtml+xml', 'text/html'];
@@ -410,7 +420,7 @@ function createDOMPurify(window = getGlobal()) {
     // HTML tags and attributes are not case-sensitive, converting to lowercase. Keeping XHTML as is.
     transformCaseFunc =
       PARSER_MEDIA_TYPE === 'application/xhtml+xml'
-        ? (x) => x
+        ? stringToString
         : stringToLowerCase;
 
     /* Set configuration parameters */
@@ -422,6 +432,10 @@ function createDOMPurify(window = getGlobal()) {
       'ALLOWED_ATTR' in cfg
         ? addToSet({}, cfg.ALLOWED_ATTR, transformCaseFunc)
         : DEFAULT_ALLOWED_ATTR;
+    ALLOWED_NAMESPACES =
+      'ALLOWED_NAMESPACES' in cfg
+        ? addToSet({}, cfg.ALLOWED_NAMESPACES, stringToString)
+        : DEFAULT_ALLOWED_NAMESPACES;
     URI_SAFE_ATTRIBUTES =
       'ADD_URI_SAFE_ATTR' in cfg
         ? addToSet(
@@ -633,7 +647,7 @@ function createDOMPurify(window = getGlobal()) {
     // can be null. We just simulate parent in this case.
     if (!parent || !parent.tagName) {
       parent = {
-        namespaceURI: HTML_NAMESPACE,
+        namespaceURI: NAMESPACE,
         tagName: 'template',
       };
     }
@@ -641,6 +655,10 @@ function createDOMPurify(window = getGlobal()) {
     const tagName = stringToLowerCase(element.tagName);
     const parentTagName = stringToLowerCase(parent.tagName);
 
+    if (!ALLOWED_NAMESPACES[element.namespaceURI]) {
+      return false;
+    }
+
     if (element.namespaceURI === SVG_NAMESPACE) {
       // The only way to switch from HTML namespace to SVG
       // is via <svg>. If it happens via any other tag, then
@@ -649,7 +667,7 @@ function createDOMPurify(window = getGlobal()) {
         return tagName === 'svg';
       }
 
-      // The only way to switch from MathML to SVG is via
+      // The only way to switch from MathML to SVG is via`
       // svg if parent is either <annotation-xml> or MathML
       // text integration points.
       if (parent.namespaceURI === MATHML_NAMESPACE) {
@@ -710,9 +728,18 @@ function createDOMPurify(window = getGlobal()) {
       );
     }
 
+    // For XHTML and XML documents that support custom namespaces
+    if (
+      PARSER_MEDIA_TYPE === 'application/xhtml+xml' &&
+      ALLOWED_NAMESPACES[element.namespaceURI]
+    ) {
+      return true;
+    }
+
     // The code should never reach this place (this means
     // that the element somehow got namespace that is not
-    // HTML, SVG or MathML). Return false just in case.
+    // HTML, SVG, MathML or allowed via ALLOWED_NAMESPACES).
+    // Return false just in case.
     return false;
   };
 
@@ -789,7 +816,10 @@ function createDOMPurify(window = getGlobal()) {
       leadingWhitespace = matches && matches[0];
     }
 
-    if (PARSER_MEDIA_TYPE === 'application/xhtml+xml') {
+    if (
+      PARSER_MEDIA_TYPE === 'application/xhtml+xml' &&
+      NAMESPACE === HTML_NAMESPACE
+    ) {
       // Root of XHTML doc must contain xmlns declaration (see https://www.w3.org/TR/xhtml1/normative.html#strict)
       dirty =
         '<html xmlns="http://www.w3.org/1999/xhtml"><head></head><body>' +
@@ -873,7 +903,8 @@ function createDOMPurify(window = getGlobal()) {
         typeof elm.removeAttribute !== 'function' ||
         typeof elm.setAttribute !== 'function' ||
         typeof elm.namespaceURI !== 'string' ||
-        typeof elm.insertBefore !== 'function')
+        typeof elm.insertBefore !== 'function' ||
+        typeof elm.hasChildNodes !== 'function')
     );
   };
 
@@ -1026,6 +1057,7 @@ function createDOMPurify(window = getGlobal()) {
       content = currentNode.textContent;
       content = stringReplace(content, MUSTACHE_EXPR, ' ');
       content = stringReplace(content, ERB_EXPR, ' ');
+      content = stringReplace(content, TMPLIT_EXPR, ' ');
       if (currentNode.textContent !== content) {
         arrayPush(DOMPurify.removed, { element: currentNode.cloneNode() });
         currentNode.textContent = content;
@@ -1216,6 +1248,7 @@ function createDOMPurify(window = getGlobal()) {
       if (SAFE_FOR_TEMPLATES) {
         value = stringReplace(value, MUSTACHE_EXPR, ' ');
         value = stringReplace(value, ERB_EXPR, ' ');
+        value = stringReplace(value, TMPLIT_EXPR, ' ');
       }
 
       /* Is `value` valid for this attribute? */
@@ -1505,6 +1538,7 @@ function createDOMPurify(window = getGlobal()) {
     if (SAFE_FOR_TEMPLATES) {
       serializedHTML = stringReplace(serializedHTML, MUSTACHE_EXPR, ' ');
       serializedHTML = stringReplace(serializedHTML, ERB_EXPR, ' ');
+      serializedHTML = stringReplace(serializedHTML, TMPLIT_EXPR, ' ');
     }
 
     return trustedTypesPolicy && RETURN_TRUSTED_TYPE
