@@ -1413,7 +1413,8 @@
           expected: '<img src="http://i.imgur.com/WScAnHr.jpg">',
         },
         {
-          test: '<img src="blob:https://localhost:3000/c4ea3ec6-9f22-4d08-af6f-d79e78a0a7a7">',
+          test:
+            '<img src="blob:https://localhost:3000/c4ea3ec6-9f22-4d08-af6f-d79e78a0a7a7">',
           expected: '<img>',
         },
         {
@@ -1423,8 +1424,7 @@
       ];
       tests.forEach(function (test) {
         var str = DOMPurify.sanitize(test.test, {
-          ALLOWED_URI_REGEXP:
-            /^(?:(?:(?:f|ht)tps?):|[^a-z]|[a-z+.\-]+(?:[^a-z+.\-:]|$))/i,
+          ALLOWED_URI_REGEXP: /^(?:(?:(?:f|ht)tps?):|[^a-z]|[a-z+.\-]+(?:[^a-z+.\-:]|$))/i,
         });
         assert.equal(str, test.expected);
       });
@@ -1704,7 +1704,8 @@
     QUnit.test('Test if namespaces are properly enforced', function (assert) {
       var tests = [
         {
-          test: '<svg><desc><canvas></canvas><textarea></textarea></desc></svg>',
+          test:
+            '<svg><desc><canvas></canvas><textarea></textarea></desc></svg>',
           expected: [
             '<svg><desc><canvas></canvas><textarea></textarea></desc></svg>',
             '<svg xmlns="http://www.w3.org/2000/svg"><desc><canvas></canvas><textarea></textarea></desc></svg>',
@@ -1802,7 +1803,97 @@
         assert.contains(clean, test.expected);
       });
     });
-
+    QUnit.test('Config-Flag tests: ALLOWED_NAMESPACES', function (assert) {
+      const tests = [
+        // Test when ALLOWED_NAMESPACES is not set, result is empty for XML with custom namespace
+        {
+          test:
+            '<library xmlns="http://www.ibm.com/library"><name>Library 1</name></library>',
+          config: {
+            ALLOWED_TAGS: ['#text', 'library', 'name'],
+            KEEP_CONTENT: false,
+            PARSER_MEDIA_TYPE: 'application/xhtml+xml',
+          },
+          expected: '',
+        },
+        // Test with one custom namespace at the root (ie. all sub-nodes will inherit that namespace)
+        {
+          test:
+            '<library xmlns="http://www.ibm.com/library"><name>Library 1</name><dirty onload="alert()" /></library>',
+          config: {
+            ALLOWED_NAMESPACES: ['http://www.ibm.com/library'],
+            ALLOWED_TAGS: ['#text', 'library', 'name'],
+            KEEP_CONTENT: false,
+            PARSER_MEDIA_TYPE: 'application/xhtml+xml',
+          },
+          expected:
+            '<library xmlns="http://www.ibm.com/library"><name>Library 1</name></library>',
+        },
+        // Test with one custom namespace at sub-node (root will default to HTML_NAMESPACE and should be kept)
+        {
+          test:
+            '<city><library xmlns="http://www.ibm.com/library"><name>Library 1</name><dirty onload="alert()" /></library></city>',
+          config: {
+            ALLOWED_NAMESPACES: ['http://www.w3.org/1999/xhtml', 'http://www.ibm.com/library'],
+            ALLOWED_TAGS: ['#text', 'city', 'library', 'name'],
+            KEEP_CONTENT: false,
+            PARSER_MEDIA_TYPE: 'application/xhtml+xml',
+          },
+          expected:
+            '<city xmlns="http://www.w3.org/1999/xhtml"><library xmlns="http://www.ibm.com/library"><name>Library 1</name></library></city>',
+        },
+        // Test removal of namespaces not listed in ALLOWED_NAMESPACES when input has multiple namespaces
+        {
+          test:
+            '<library xmlns="http://www.ibm.com/library" xmlns:bk="urn:loc.gov:books"><bk:name>Library 1</bk:name><dirty onload="alert()" /></library>',
+          config: {
+            ALLOWED_NAMESPACES: ['http://www.ibm.com/library'],
+            ALLOWED_TAGS: ['library', 'bk:name'],
+            KEEP_CONTENT: false,
+            PARSER_MEDIA_TYPE: 'application/xhtml+xml',
+          },
+          expected: ['<library xmlns="http://www.ibm.com/library"/>', '<library xmlns="http://www.ibm.com/library" />']
+        },
+        // Test with multiple custom namespaces and prefixes in input
+        {
+          test:
+            '<library xmlns="http://www.ibm.com/library" xmlns:bk="urn:loc.gov:books" xmlns:m="http://schemas.microsoft.com/ado/2007/08/dataservices/metadata"><bk:name>Library 1<m:properties>Other Properties</m:properties></bk:name><dirty onload="alert()" /></library>',
+          config: {
+            ALLOWED_NAMESPACES: [
+              'http://www.ibm.com/library',
+              'urn:loc.gov:books',
+              'http://schemas.microsoft.com/ado/2007/08/dataservices/metadata',
+            ],
+            ALLOWED_TAGS: ['#text', 'library', 'bk:name', 'm:properties'],
+            KEEP_CONTENT: false,
+            PARSER_MEDIA_TYPE: 'application/xhtml+xml',
+          },
+          expected:
+            '<library xmlns="http://www.ibm.com/library"><bk:name xmlns:bk="urn:loc.gov:books">Library 1<m:properties xmlns:m="http://schemas.microsoft.com/ado/2007/08/dataservices/metadata">Other Properties</m:properties></bk:name></library>',
+        },
+        // Test removal of elements mentioned in FORBID_TAGS even if their namespaces are allow-listed
+        {
+          test:
+            '<library xmlns="http://www.ibm.com/library" xmlns:bk="urn:loc.gov:books" xmlns:m="http://schemas.microsoft.com/ado/2007/08/dataservices/metadata"><bk:name>Library 1<m:properties>Other Properties</m:properties></bk:name><dirty onload="alert()" /></library>',
+          config: {
+            ADD_TAGS: ['library', 'bk:name'],
+            ALLOWED_NAMESPACES: [
+              'http://www.ibm.com/library',
+              'urn:loc.gov:books',
+              'http://schemas.microsoft.com/ado/2007/08/dataservices/metadata',
+            ],
+            FORBID_TAGS: ['m:properties'],
+            KEEP_CONTENT: false,
+            PARSER_MEDIA_TYPE: 'application/xhtml+xml',
+          },
+          expected:
+            '<library xmlns="http://www.ibm.com/library"><bk:name xmlns:bk="urn:loc.gov:books">Library 1</bk:name></library>',
+        },
+      ];
+      tests.forEach(function (test) {
+        assert.contains(DOMPurify.sanitize(test.test, test.config), test.expected);
+      });
+    });
     QUnit.test('Config-Flag tests: PARSER_MEDIA_TYPE', function (assert) {
       const tests = [
         {
